
#include "system.h"
#include "stdio.h"
#include "io.h"
#include "altera_up_avalon_ps2.h"
#include "altera_up_ps2_keyboard.h"

void read_keyboard(alt_up_ps2_dev *ps2_kb)

{
	struct button
		{
			alt_u8 reg_n;
			alt_u8 reg_b;
		};

	struct button * conv_table;
	int conv_table_change = 0;

	struct button conv_table_l[0x87] =
			{
				{1, 2},	//00
				{1, 3},	//01
				{2, 4},	//02
				{3, 4},	//03
				{3, 4},	//04
				{3, 4},	//05
				{3, 4},	//06
				{3, 4},	//07
				{3, 4},	//08
				{3, 4},	//09
				{3, 4},	//0A
				{3, 4},	//0B
				{3, 4},	//0C
				{1, 0x01},	//0D Tab
				{3, 4},	//0E
				{3, 4},	//0F
				{3, 4},	//10
				{3, 4},	//11
				{3, 4},	//12
				{3, 4},	//13
				{3, 4},	//14
				{6, 0x02},	//15 Q
				{2, 0x02},	//16 1
				{3, 4},	//17
				{3, 4},	//18
				{3, 4},	//19
				{7, 0x04},	//1A Z
				{6, 0x08},	//1B S
				{4, 0x02},	//1C A
				{6, 0x80},	//1D W
				{2, 0x04},	//1E 2
				{3, 4},	//1F
				{3, 4},	//20
				{4, 0x08},	//21 C
				{7, 0x01},	//22 X
				{4, 0x10},	//23 D
				{4, 0x20},	//24 E
				{2, 0x10},	//25 4
				{2, 0x08},	//26 3
				{3, 4},	//27
				{3, 4},	//28
				{7, 0x80},	//29 Space
				{6, 0x40},	//2A V
				{4, 0x40},	//2B F
				{6, 0x10},	//2C T
				{6, 0x04},	//2D R
				{2, 0x20},	//2E 5
				{3, 4},	//2F
				{3, 4},	//30
				{5, 0x40},	//31 N
				{4, 0x04},	//32 B
				{5, 0x01},	//33 H
				{4, 0x80},	//34 G
				{7, 0x02},	//35 Y
				{2, 0x40},	//36 6
				{3, 4},	//37
				{3, 4},	//38
				{3, 4},	//39
				{5, 0x20},	//3A M
				{5, 0x04},	//3B J
				{6, 0x20},	//3C U
				{2, 0x80},	//3D 7
				{3, 0x01},	//3E 8
				{3, 4},	//3F
				{3, 4},	//40
				{3, 0x10},	//41 ,
				{5, 0x08},	//42 K
				{5, 0x02},	//43 I
				{5, 0x80},	//44 O
				{2, 0x01},	//45 0
				{3, 0x02},	//46 9
				{3, 4},	//47
				{3, 4},	//48
				{3, 0x40},	//49 .
				{3, 4},	//4A
				{5, 0x10},	//4B L
				{3, 4},	//4C
				{6, 0x01},	//4D P
				{3, 0x20},	//4E -
				{3, 4},	//4F
				{3, 4},	//50
				{3, 4},	//51
				{3, 4},	//52
				{3, 4},	//53
				{3, 4},	//54
				{3, 8},	//55 +
				{3, 4},	//56
				{3, 4},	//57
				{3, 4},	//58
				{3, 4},	//59
				{1, 0x04},	//5A Enter
				{3, 4},	//5B
				{3, 4},	//5C
				{3, 4},	//5D
				{3, 4},	//5E
				{3, 4},	//5F
				{3, 4},	//60
				{3, 4},	//61
				{3, 4},	//62
				{3, 4},	//63
				{3, 4},	//64
				{3, 4},	//65
				{1, 0x08},	//66 Backspace
				{3, 4},	//67
				{3, 4},	//68
				{3, 4},	//69
				{3, 4},	//6A
				{1, 0x10},	//E0 6B Left
				{3, 4},	//6C
				{3, 4},	//6D
				{3, 4},	//6E
				{3, 4},	//6F
				{3, 4},	//70
				{3, 4},	//71
				{1, 0x80},	//E0 72 Down
				{3, 4},	//73
				{1, 0x40},	//E0 74 Right
				{1, 0x20},	//E0 75 Up
				{3, 4},	//76
				{3, 4},	//77
				{3, 4},	//78
				{3, 0x08},	//79 Right +
				{3, 4},	//7A
				{3, 0x20},	//7B Right -
				{3, 0x04},	//7C *
				{3, 4},	//7D
				{3, 4},	//7E
				{3, 4},	//7F
				{3, 4},	//80
				{3, 4},	//81
				{3, 4},	//82
				{3, 4},	//83
				{3, 4},	//84
				{3, 4},	//85
				{3, 4},	//86
			};

	struct button conv_table_r[0x87] =
				{
					{1, 2},	//00
					{1, 3},	//01
					{2, 4},	//02
					{3, 4},	//03
					{3, 4},	//04
					{3, 4},	//05
					{3, 4},	//06
					{3, 4},	//07
					{3, 4},	//08
					{3, 4},	//09
					{3, 4},	//0A
					{3, 4},	//0B
					{3, 4},	//0C
					{1, 0x01},	//0D Tab
					{3, 4},	//0E
					{3, 4},	//0F
					{3, 4},	//10
					{3, 4},	//11
					{3, 4},	//12
					{3, 4},	//13
					{3, 4},	//14
					{5, 0x04},	//15 Q - J (Й)
					{2, 0x02},	//16 1
					{3, 4},	//17
					{3, 4},	//18
					{3, 4},	//19
					{7, 0x04},	//1A Z
					{6, 0x08},	//1B S
					{4, 0x02},	//1C A
					{6, 0x80},	//1D W
					{2, 0x04},	//1E 2
					{3, 4},	//1F
					{3, 4},	//20
					{4, 0x08},	//21 C
					{7, 0x01},	//22 X
					{4, 0x10},	//23 D
					{4, 0x20},	//24 E
					{2, 0x10},	//25 4
					{2, 0x08},	//26 3
					{3, 4},	//27
					{3, 4},	//28
					{7, 0x80},	//29 Space
					{6, 0x40},	//2A V
					{4, 0x40},	//2B F
					{6, 0x10},	//2C T
					{6, 0x04},	//2D R
					{2, 0x20},	//2E 5
					{3, 4},	//2F
					{3, 4},	//30
					{5, 0x40},	//31 N
					{4, 0x04},	//32 B
					{5, 0x01},	//33 H
					{4, 0x80},	//34 G
					{7, 0x02},	//35 Y
					{2, 0x40},	//36 6
					{3, 4},	//37
					{3, 4},	//38
					{3, 4},	//39
					{5, 0x20},	//3A M
					{5, 0x04},	//3B J
					{6, 0x20},	//3C U
					{2, 0x80},	//3D 7
					{3, 0x01},	//3E 8
					{3, 4},	//3F
					{3, 4},	//40
					{3, 4},	//41
					{5, 0x08},	//42 K
					{5, 0x02},	//43 I
					{5, 0x80},	//44 O
					{3, 0x02},	//45 0 - )
					{3, 0x01},	//46 9 - (
					{3, 4},	//47
					{3, 4},	//48
					{3, 4},	//49
					{3, 4},	//4A
					{5, 0x10},	//4B L
					{3, 4},	//4C
					{6, 0x01},	//4D P
					{3, 0x20},	//4E
					{3, 4},	//4F
					{3, 4},	//50
					{3, 4},	//51
					{3, 4},	//52
					{3, 4},	//53
					{3, 4},	//54
					{3, 8},	//55
					{3, 4},	//56
					{3, 4},	//57
					{3, 4},	//58
					{3, 4},	//59
					{1, 0x04},	//5A Enter
					{3, 4},	//5B
					{3, 4},	//5C
					{3, 4},	//5D
					{3, 4},	//5E
					{3, 4},	//5F
					{3, 4},	//60
					{3, 4},	//61
					{3, 4},	//62
					{3, 4},	//63
					{3, 4},	//64
					{3, 4},	//65
					{1, 0x08},	//66 Backspace
					{3, 4},	//67
					{3, 4},	//68
					{3, 4},	//69
					{3, 4},	//6A
					{1, 0x10},	//E0 6B Left
					{3, 4},	//6C
					{3, 4},	//6D
					{3, 4},	//6E
					{3, 4},	//6F
					{3, 4},	//70
					{3, 4},	//71
					{1, 0x80},	//E0 72 Down
					{3, 4},	//73
					{1, 0x40},	//E0 74 Right
					{1, 0x20},	//E0 75 Up
					{3, 4},	//76
					{3, 4},	//77
					{3, 4},	//78
					{3, 4},	//79
					{3, 4},	//7A
					{3, 4},	//7B
					{3, 0x04},	//7C *
					{3, 4},	//7D
					{3, 4},	//7E
					{3, 4},	//7F
					{3, 4},	//80
					{3, 4},	//81
					{3, 4},	//82
					{3, 4},	//83
					{3, 4},	//84
					{3, 4},	//85
					{3, 4},	//86
				};

	alt_u32 regs[8] = {R0_BASE, R1_BASE, R2_BASE, R3_BASE, R4_BASE, R5_BASE, R6_BASE, R7_BASE};

	unsigned char scan;
	ps2_kb->timeout = 0;
	int brk = 0;
	alt_u8 port_c = 0xff;

	alt_u8 register_n, register_bit, register_bit_n;

	alt_u8 registers[8] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	conv_table = conv_table_l;

	while (1)
	{
		if (alt_up_ps2_read_data_byte (ps2_kb, &scan) == 0)
		{
			switch (scan)
			{
			case 0xf0 :		// Break
				brk = 1;
				break;

			case 0xe0 :		//Long break
				break;

			case 0x12 :		//Left Shift
				if (brk == 0)
				{
					port_c = port_c & 6;
					IOWR(PORT_C567_OUT_BASE, 0, port_c);
					if (conv_table == conv_table_l)
					{
						conv_table = conv_table_r;
						conv_table_change = 1;
					}
				}

				else
				{
					brk = 0;
					conv_table_change = 0;
					port_c = port_c | 1;
					IOWR(PORT_C567_OUT_BASE, 0, port_c);
					conv_table = conv_table_l;
				}
				break;

			case 0x14 :		// Left Ctrl - RUS/LAT
				if (brk == 0)
				{
					port_c = port_c & 3;
					IOWR(PORT_C567_OUT_BASE, 0, port_c);
				}

				else
				{
					brk = 0;
					port_c = port_c | 4;
					IOWR(PORT_C567_OUT_BASE, 0, port_c);

					if (conv_table == conv_table_r)
					{
							conv_table = conv_table_l;
							ResetCapsLock (ps2_kb);
					}

					else
					{
							conv_table = conv_table_r;
							SetCapsLock (ps2_kb);
					}
				}
				break;

			default :
				register_n = conv_table[scan].reg_n;
				register_bit = conv_table[scan].reg_b;
				register_bit_n = ~register_bit;

				if (brk == 0)
				{
					registers[register_n] = registers[register_n] & register_bit_n;
					IOWR(regs[register_n], 0, registers[register_n]);
					printf ("%x Set %x  %x\n",scan, register_n, register_bit);
				}

				else
				{
					brk = 0;
					registers[register_n] = registers[register_n] | register_bit;
					IOWR(regs[register_n], 0, registers[register_n]);
					printf ("%x Reset %x  %x\n",scan, register_n, register_bit);
				}
				break;
			}
		}
	}
}


void SetCapsLock (alt_up_ps2_dev *ps2_kb)
{
	char scan;

	alt_up_ps2_write_data_byte (ps2_kb, 0xed);
	while (alt_up_ps2_read_data_byte (ps2_kb, &scan) != 0) { }

	alt_up_ps2_write_data_byte (ps2_kb, 0x02);
	while (alt_up_ps2_read_data_byte (ps2_kb, &scan) != 0) { }
}

void ResetCapsLock (alt_up_ps2_dev *ps2_kb)
{
	char scan;

	alt_up_ps2_write_data_byte (ps2_kb, 0xed);
	while (alt_up_ps2_read_data_byte (ps2_kb, &scan) != 0) { }

	alt_up_ps2_write_data_byte (ps2_kb, 0x00);
	while (alt_up_ps2_read_data_byte (ps2_kb, &scan) != 0) { }
}


int main (void)

{
	alt_u32 regs[8] = {R0_BASE, R1_BASE, R2_BASE, R3_BASE, R4_BASE, R5_BASE, R6_BASE, R7_BASE};

	int count;

	for (count = 0; count < 8; count++) IOWR(regs[count], 0, 0xff);

	printf("Starting\n");

	alt_up_ps2_dev *ps2 = alt_up_ps2_open_dev(PS2_NAME); //open PS/2 device

	printf("PS2_NAME opened\n");

    alt_up_ps2_init (ps2);
    alt_up_ps2_disable_read_interrupt (ps2);
    //reset_keyboard (ps2);

    printf("PS2 initialized\n");

    alt_up_ps2_clear_fifo(ps2);

    printf("FIFO cleared\n");

    if (ps2->device_type == PS2_KEYBOARD)
    	{
    		printf("Keyboard found\n");
    		read_keyboard(ps2);
    	}
    else
    	{
        	printf("Keyboard not found\n");

        	while (1)
        	{

        	}

        	return -1;
    	}

    return 0;
}

